@@ -19,7 +19,6 @@ macro_rules! const_key {
 }
 
 // we define a few constants which make it easier to check key events
-//const_key!(ALT_ENTER, Enter, KeyModifiers::ALT);
 const_key!(ENTER, Enter);
 //const_key!(BACKSPACE, Backspace);
 //const_key!(BACK_TAB, BackTab, KeyModifiers::SHIFT); // backtab needs shift
@@ -38,13 +37,18 @@ const_key!(TAB, Tab);
 const_key!(UP, Up);
 const_key!(PAGE_UP, PageUp);
 //const_key!(F1, F(1));
+const_key!(CONTROL_C, Char('c'), KeyModifiers::CONTROL);
 const_key!(CONTROL_H, Char('h'), KeyModifiers::CONTROL);
+const_key!(CONTROL_N, Char('n'), KeyModifiers::CONTROL);
+const_key!(CONTROL_O, Char('o'), KeyModifiers::CONTROL);
 const_key!(CONTROL_Q, Char('q'), KeyModifiers::CONTROL);
 const_key!(CONTROL_S, Char('s'), KeyModifiers::CONTROL);
 const_key!(CONTROL_X, Char('x'), KeyModifiers::CONTROL);
 const_key!(CONTROL_UP, Up, KeyModifiers::CONTROL);
 const_key!(CONTROL_DOWN, Down, KeyModifiers::CONTROL);
 
+
+
 /// return the raw char if the event is a letter event
 pub fn as_letter(key: KeyEvent) -> Option<char> {
     match key {
